mod mdtransform;
use mdtransform::MdTransform;
use poem::{
    error::BadRequest, listener::TcpListener, middleware::AddData, web::Data, EndpointExt, Route,
};
use poem_openapi::{param::Query, payload::PlainText, OpenApi, OpenApiService};
use sqlx::{postgres::PgPoolOptions, Error, Pool, Postgres};
use tokio::{
    fs::{self, File},
    sync::Mutex,
};

struct Api {
    pool: Mutex<Pool<Postgres>>,
}

#[OpenApi]
impl Api {
    #[oai(path = "/hello", method = "get")]
    async fn index(&self, name: Query<Option<String>>) -> PlainText<String> {
        match name.0 {
            Some(name) => {
                let pool = &self.pool.lock().await;
                insertSomething(pool, &name).await;
                return PlainText(format!("Inserted article with name: {}", name).to_string());
            }
            None => PlainText("hello!".to_string()),
        }
    }
}

async fn insertSomething(pool: &Pool<Postgres>, name: &String) {
    sqlx::query!(
        "INSERT INTO articles (markdown, name) VALUES ($1, $2);",
        "ARTICLE HERE",
        name
    )
    .execute(pool)
    .await
    .unwrap();
}

#[tokio::main]
async fn main() -> Result<(), std::io::Error> {
    let pool = PgPoolOptions::new()
        .max_connections(5)
        .connect("postgres://postgres:localdb@localhost:5432/postgres")
        .await
        .unwrap();

    let api_service = OpenApiService::new(
        Api {
            pool: Mutex::new(pool),
        },
        "Hello World",
        "1.0",
    )
    .server("http://localhost:3000/api");
    let ui = api_service.swagger_ui();
    let app = Route::new().nest("/api", api_service).nest("/", ui);

    let file = fs::read_to_string("test.md").await.unwrap();
    println!("{}", MdTransform::transform(file));

    println!("HELLO!");
    poem::Server::new(TcpListener::bind("127.0.0.1:3000"))
        .run(app)
        .await
}
